
/*******************************************************************************
* Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
*
* This file is part of PAPPSOms-tools.
*
*     PAPPSOms-tools is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     PAPPSOms-tools is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with PAPPSOms-tools.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
******************************************************************************/
#ifndef SPECTRUMWINDOW_H
#define SPECTRUMWINDOW_H


#include <QMainWindow>
#include <QThread>
#include <QFileInfo>
#include <QMutex>
#include <pappsomspp/types.h>
#include <pappsomspp/msrun/msrunid.h>
#include <pappsomspp/peptide/peptide.h>
#include <pappsomspp/spectrum/qualifiedspectrum.h>
#include <pwiz/data/msdata/MSDataFile.hpp>
#include "../utils/readspectrum.h"
#include "overlaywidget.h"
#include <ui_spectrumwindow.h>


namespace Ui {
class SpectrumWindow;
}


class PwizLoaderThread : public QObject
{
    Q_OBJECT
public:
    bool isLoading() {
        if (_mutex.tryLock()) {
            _mutex.unlock();
            return false;
        }
        return true;
    }

public slots:
    void doMsDataFileLoad(const QString & parameter) {
        if (isLoading()) return;
        _mutex.lock();
        pwiz::msdata::MSDataFile* p_ms_data_file = getPwizMSDataFile(parameter);
        /* ... here is the expensive or blocking operation ... */
        _mutex.unlock();
        emit msDataReady(p_ms_data_file);
    }

signals:
    void msDataReady(pwiz::msdata::MSDataFile * p_ms_data_file);

private :
    QMutex _mutex;
};

class SpectrumWindow: public QMainWindow {
    Q_OBJECT
    QThread workerThread;

public:

    explicit SpectrumWindow(QWidget * parent = 0);
    ~SpectrumWindow();
    
    QSpectrumWidget * getQSpectrumWidget() {
      return (ui->spectrumWidget);
    };
    
    void setMzDataFile(const QString & filepath);

public slots:
    void setPeptideSp(PeptideSp peptide_sp);
    void selectMzFile();
    void saveSvgFile();
    void setMz(double mz);
    void setScanNum(int scan_num);
    void handleMsDataFile(pwiz::msdata::MSDataFile * p_ms_data_file);
    void setPrecision(PrecisionP p_precision);
    void setFilteredSpectrumSp(SpectrumSp filtered_spectrum);
    void setIsotopeMassList(std::vector<pappso::PeptideNaturalIsotopeAverageSp> isotopeMassList);
    // void setColor(const QColor &color);
    // void setShape(Shape shape);
signals:
    void spectrumChanged (pappso::QualifiedSpectrum spectrum);
    void operateMsDataFile(const QString & parameter);
    void mzChanged(double mz);
private:
    Ui::SpectrumWindow *ui;

    QFileInfo _mz_file_info;
    pwiz::msdata::MSDataFile* _p_ms_data_file=nullptr;
    LoadingOverlay* _p_spectrum_overlay;

    QualifiedSpectrum _spectrum;
    pappso::MsRunId _msrun_id;

    PwizLoaderThread *_worker_loader;
};

#endif // SPECTRUMWINDOW_H

